cmake_policy(PUSH)
if(POLICY CMP0057)
  cmake_policy(SET CMP0057 NEW) # IN_LIST operator
endif()

# CMake выполняет проверку системного окружения с помощью модулей,
# расположенных в следующих каталогах:
# /usr/share/cmake-${CMAKE_VERSION}/Modules
# /usr/lib/${CMAKE_LIBRARY_ARCHITECTURE}/cmake
# /usr/lib/cmake
# Если для используемой программы или библиотеки нет стандартного
# модуля для поиска, то можно использовать собственный.
# С помощью переменной `CMAKE_MODULE_PATH` указывается перечень
# дополнительных каталогов, в которых производится поиск модулей.
set(MYX_CMAKE_FIND_DIR "${PROJECT_SOURCE_DIR}/cmake/find")
if(IS_DIRECTORY "${MYX_CMAKE_FIND_DIR}")
  if(NOT ${MYX_CMAKE_FIND_DIR} IN_LIST CMAKE_MODULE_PATH)
    list(INSERT CMAKE_MODULE_PATH 0 "${MYX_CMAKE_FIND_DIR}")
  endif()
endif()

get_filename_component(MYX_CMAKE_SOURCE_DIR "${CMAKE_CURRENT_LIST_FILE}" DIRECTORY)

set(MYX_CMAKE_BACKPORTS_DIR "${MYX_CMAKE_SOURCE_DIR}/backports")
set(MYX_CMAKE_LIB_DIR "${MYX_CMAKE_SOURCE_DIR}/lib")

# Модули для обеспечения обратной совместимости со старыми версиями CMake
# Защита для однократного включения файла *.cmake
# Функция include_guard() реализована в версии 3.10
# Макрос реализован для обратной совместимости
if(${CMAKE_VERSION} VERSION_LESS "3.10.0")
  macro(include_guard)
    if(CMAKE_FILE_${CMAKE_CURRENT_LIST_FILE}_ALREADY_INCLUDED)
      return()
    endif()
    set(CMAKE_FILE_${CMAKE_CURRENT_LIST_FILE}_ALREADY_INCLUDED TRUE)
  endmacro()
endif()


if(${CMAKE_VERSION} VERSION_LESS 3.11)
include_guard(GLOBAL)

set(__FetchContent_privateDir "${CMAKE_CURRENT_LIST_DIR}/FetchContent")

#=======================================================================
# Recording and retrieving content details for later population
#=======================================================================

# Internal use, projects must not call this directly. It is
# intended for use by FetchContent_Declare() only.
#
# Sets a content-specific global property (not meant for use
# outside of functions defined here in this file) which can later
# be retrieved using __FetchContent_getSavedDetails() with just the
# same content name. If there is already a value stored in the
# property, it is left unchanged and this call has no effect.
# This allows parent projects to define the content details,
# overriding anything a child project may try to set (properties
# are not cached between runs, so the first thing to set it in a
# build will be in control).
function(__FetchContent_declareDetails contentName)

  string(TOLOWER ${contentName} contentNameLower)
  set(propertyName "_FetchContent_${contentNameLower}_savedDetails")
  get_property(alreadyDefined GLOBAL PROPERTY ${propertyName} DEFINED)
  if(NOT alreadyDefined)
    define_property(GLOBAL PROPERTY ${propertyName}
      BRIEF_DOCS "Internal implementation detail of FetchContent_Populate()"
      FULL_DOCS  "Details used by FetchContent_Populate() for ${contentName}"
    )
    set_property(GLOBAL PROPERTY ${propertyName} ${ARGN})
  endif()

endfunction()


# Internal use, projects must not call this directly. It is
# intended for use by the FetchContent_Declare() function.
#
# Retrieves details saved for the specified content in an
# earlier call to __FetchContent_declareDetails().
function(__FetchContent_getSavedDetails contentName outVar)

  string(TOLOWER ${contentName} contentNameLower)
  set(propertyName "_FetchContent_${contentNameLower}_savedDetails")
  get_property(alreadyDefined GLOBAL PROPERTY ${propertyName} DEFINED)
  if(NOT alreadyDefined)
    message(FATAL_ERROR "No content details recorded for ${contentName}")
  endif()
  get_property(propertyValue GLOBAL PROPERTY ${propertyName})
  set(${outVar} "${propertyValue}" PARENT_SCOPE)

endfunction()


# Saves population details of the content, sets defaults for the
# SOURCE_DIR and BUILD_DIR.
function(FetchContent_Declare contentName)

  set(options "")
  set(oneValueArgs SVN_REPOSITORY)
  set(multiValueArgs "")

  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  unset(srcDirSuffix)
  unset(svnRepoArgs)
  if(ARG_SVN_REPOSITORY)
    # Add a hash of the svn repository URL to the source dir. This works
    # around the problem where if the URL changes, the download would
    # fail because it tries to checkout/update rather than switch the
    # old URL to the new one. We limit the hash to the first 7 characters
    # so that the source path doesn't get overly long (which can be a
    # problem on windows due to path length limits).
    string(SHA1 urlSHA ${ARG_SVN_REPOSITORY})
    string(SUBSTRING ${urlSHA} 0 7 urlSHA)
    set(srcDirSuffix "-${urlSHA}")
    set(svnRepoArgs  SVN_REPOSITORY ${ARG_SVN_REPOSITORY})
  endif()

  string(TOLOWER ${contentName} contentNameLower)
  __FetchContent_declareDetails(
    ${contentNameLower}
    SOURCE_DIR "${FETCHCONTENT_BASE_DIR}/${contentNameLower}-src${srcDirSuffix}"
    BINARY_DIR "${FETCHCONTENT_BASE_DIR}/${contentNameLower}-build"
    ${svnRepoArgs}
    # List these last so they can override things we set above
    ${ARG_UNPARSED_ARGUMENTS}
  )

endfunction()


#=======================================================================
# Set/get whether the specified content has been populated yet.
# The setter also records the source and binary dirs used.
#=======================================================================

# Internal use, projects must not call this directly. It is
# intended for use by the FetchContent_Populate() function to
# record when FetchContent_Populate() is called for a particular
# content name.
function(__FetchContent_setPopulated contentName sourceDir binaryDir)

  string(TOLOWER ${contentName} contentNameLower)
  set(prefix "_FetchContent_${contentNameLower}")

  set(propertyName "${prefix}_sourceDir")
  define_property(GLOBAL PROPERTY ${propertyName}
    BRIEF_DOCS "Internal implementation detail of FetchContent_Populate()"
    FULL_DOCS  "Details used by FetchContent_Populate() for ${contentName}"
  )
  set_property(GLOBAL PROPERTY ${propertyName} ${sourceDir})

  set(propertyName "${prefix}_binaryDir")
  define_property(GLOBAL PROPERTY ${propertyName}
    BRIEF_DOCS "Internal implementation detail of FetchContent_Populate()"
    FULL_DOCS  "Details used by FetchContent_Populate() for ${contentName}"
  )
  set_property(GLOBAL PROPERTY ${propertyName} ${binaryDir})

  set(propertyName "${prefix}_populated")
  define_property(GLOBAL PROPERTY ${propertyName}
    BRIEF_DOCS "Internal implementation detail of FetchContent_Populate()"
    FULL_DOCS  "Details used by FetchContent_Populate() for ${contentName}"
  )
  set_property(GLOBAL PROPERTY ${propertyName} True)

endfunction()


# Set variables in the calling scope for any of the retrievable
# properties. If no specific properties are requested, variables
# will be set for all retrievable properties.
#
# This function is intended to also be used by projects as the canonical
# way to detect whether they should call FetchContent_Populate()
# and pull the populated source into the build with add_subdirectory(),
# if they are using the populated content in that way.
function(FetchContent_GetProperties contentName)

  string(TOLOWER ${contentName} contentNameLower)

  set(options "")
  set(oneValueArgs SOURCE_DIR BINARY_DIR POPULATED)
  set(multiValueArgs "")

  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  if(NOT ARG_SOURCE_DIR AND
     NOT ARG_BINARY_DIR AND
     NOT ARG_POPULATED)
    # No specific properties requested, provide them all
    set(ARG_SOURCE_DIR ${contentNameLower}_SOURCE_DIR)
    set(ARG_BINARY_DIR ${contentNameLower}_BINARY_DIR)
    set(ARG_POPULATED  ${contentNameLower}_POPULATED)
  endif()

  set(prefix "_FetchContent_${contentNameLower}")

  if(ARG_SOURCE_DIR)
    set(propertyName "${prefix}_sourceDir")
    get_property(value GLOBAL PROPERTY ${propertyName})
    if(value)
      set(${ARG_SOURCE_DIR} ${value} PARENT_SCOPE)
    endif()
  endif()

  if(ARG_BINARY_DIR)
    set(propertyName "${prefix}_binaryDir")
    get_property(value GLOBAL PROPERTY ${propertyName})
    if(value)
      set(${ARG_BINARY_DIR} ${value} PARENT_SCOPE)
    endif()
  endif()

  if(ARG_POPULATED)
    set(propertyName "${prefix}_populated")
    get_property(value GLOBAL PROPERTY ${propertyName} DEFINED)
    set(${ARG_POPULATED} ${value} PARENT_SCOPE)
  endif()

endfunction()


#=======================================================================
# Performing the population
#=======================================================================

# The value of contentName will always have been lowercased by the caller.
# All other arguments are assumed to be options that are understood by
# ExternalProject_Add(), except for QUIET and SUBBUILD_DIR.
function(__FetchContent_directPopulate contentName)

  set(options
      QUIET
  )
  set(oneValueArgs
      SUBBUILD_DIR
      SOURCE_DIR
      BINARY_DIR
      # We need special processing if DOWNLOAD_NO_EXTRACT is true
      DOWNLOAD_NO_EXTRACT
      # Prevent the following from being passed through
      CONFIGURE_COMMAND
      BUILD_COMMAND
      INSTALL_COMMAND
      TEST_COMMAND
      # We force both of these to be ON since we are always executing serially
      # and we want all steps to have access to the terminal in case they
      # need input from the command line (e.g. ask for a private key password)
      # or they want to provide timely progress. We silently absorb and
      # discard these if they are set by the caller.
      USES_TERMINAL_DOWNLOAD
      USES_TERMINAL_UPDATE
  )
  set(multiValueArgs "")

  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  if(NOT ARG_SUBBUILD_DIR)
    message(FATAL_ERROR "Internal error: SUBBUILD_DIR not set")
  elseif(NOT IS_ABSOLUTE "${ARG_SUBBUILD_DIR}")
    set(ARG_SUBBUILD_DIR "${CMAKE_CURRENT_BINARY_DIR}/${ARG_SUBBUILD_DIR}")
  endif()

  if(NOT ARG_SOURCE_DIR)
    message(FATAL_ERROR "Internal error: SOURCE_DIR not set")
  elseif(NOT IS_ABSOLUTE "${ARG_SOURCE_DIR}")
    set(ARG_SOURCE_DIR "${CMAKE_CURRENT_BINARY_DIR}/${ARG_SOURCE_DIR}")
  endif()

  if(NOT ARG_BINARY_DIR)
    message(FATAL_ERROR "Internal error: BINARY_DIR not set")
  elseif(NOT IS_ABSOLUTE "${ARG_BINARY_DIR}")
    set(ARG_BINARY_DIR "${CMAKE_CURRENT_BINARY_DIR}/${ARG_BINARY_DIR}")
  endif()

  # Ensure the caller can know where to find the source and build directories
  # with some convenient variables. Doing this here ensures the caller sees
  # the correct result in the case where the default values are overridden by
  # the content details set by the project.
  set(${contentName}_SOURCE_DIR "${ARG_SOURCE_DIR}" PARENT_SCOPE)
  set(${contentName}_BINARY_DIR "${ARG_BINARY_DIR}" PARENT_SCOPE)

  # The unparsed arguments may contain spaces, so build up ARG_EXTRA
  # in such a way that it correctly substitutes into the generated
  # CMakeLists.txt file with each argument quoted.
  unset(ARG_EXTRA)
  foreach(arg IN LISTS ARG_UNPARSED_ARGUMENTS)
    set(ARG_EXTRA "${ARG_EXTRA} \"${arg}\"")
  endforeach()

  if(ARG_DOWNLOAD_NO_EXTRACT)
    set(ARG_EXTRA "${ARG_EXTRA} DOWNLOAD_NO_EXTRACT YES")
    set(__FETCHCONTENT_COPY_FILE
"
ExternalProject_Get_Property(${contentName}-populate DOWNLOADED_FILE)
get_filename_component(dlFileName \"\${DOWNLOADED_FILE}\" NAME)

ExternalProject_Add_Step(${contentName}-populate copyfile
  COMMAND    \"${CMAKE_COMMAND}\" -E copy_if_different
             \"<DOWNLOADED_FILE>\" \"${ARG_SOURCE_DIR}\"
  DEPENDEES  patch
  DEPENDERS  configure
  BYPRODUCTS \"${ARG_SOURCE_DIR}/\${dlFileName}\"
  COMMENT    \"Copying file to SOURCE_DIR\"
)
")
  else()
    unset(__FETCHCONTENT_COPY_FILE)
  endif()

  # Hide output if requested, but save it to a variable in case there's an
  # error so we can show the output upon failure. When not quiet, don't
  # capture the output to a variable because the user may want to see the
  # output as it happens (e.g. progress during long downloads). Combine both
  # stdout and stderr in the one capture variable so the output stays in order.
  if (ARG_QUIET)
    set(outputOptions
        OUTPUT_VARIABLE capturedOutput
        ERROR_VARIABLE  capturedOutput
    )
  else()
    set(capturedOutput)
    set(outputOptions)
    message(STATUS "Populating ${contentName}")
  endif()

  if(CMAKE_GENERATOR)
    set(subCMakeOpts "-G${CMAKE_GENERATOR}")
    if(CMAKE_GENERATOR_PLATFORM)
      list(APPEND subCMakeOpts "-A${CMAKE_GENERATOR_PLATFORM}")
    endif()
    if(CMAKE_GENERATOR_TOOLSET)
      list(APPEND subCMakeOpts "-T${CMAKE_GENERATOR_TOOLSET}")
    endif()

    if(CMAKE_MAKE_PROGRAM)
      list(APPEND subCMakeOpts "-DCMAKE_MAKE_PROGRAM:FILEPATH=${CMAKE_MAKE_PROGRAM}")
    endif()

  else()
    # Likely we've been invoked via CMake's script mode where no
    # generator is set (and hence CMAKE_MAKE_PROGRAM could not be
    # trusted even if provided). We will have to rely on being
    # able to find the default generator and build tool.
    unset(subCMakeOpts)
  endif()

  # Create and build a separate CMake project to carry out the population.
  # If we've already previously done these steps, they will not cause
  # anything to be updated, so extra rebuilds of the project won't occur.
  # Make sure to pass through CMAKE_MAKE_PROGRAM in case the main project
  # has this set to something not findable on the PATH.
  configure_file("${__FetchContent_privateDir}/CMakeLists.cmake.in"
                 "${ARG_SUBBUILD_DIR}/CMakeLists.txt")
  execute_process(
    COMMAND ${CMAKE_COMMAND} ${subCMakeOpts} .
    RESULT_VARIABLE result
    ${outputOptions}
    WORKING_DIRECTORY "${ARG_SUBBUILD_DIR}"
  )
  if(result)
    if(capturedOutput)
      message("${capturedOutput}")
    endif()
    message(FATAL_ERROR "CMake step for ${contentName} failed: ${result}")
  endif()
  execute_process(
    COMMAND ${CMAKE_COMMAND} --build .
    RESULT_VARIABLE result
    ${outputOptions}
    WORKING_DIRECTORY "${ARG_SUBBUILD_DIR}"
  )
  if(result)
    if(capturedOutput)
      message("${capturedOutput}")
    endif()
    message(FATAL_ERROR "Build step for ${contentName} failed: ${result}")
  endif()

endfunction()


option(FETCHCONTENT_FULLY_DISCONNECTED   "Disables all attempts to download or update content and assumes source dirs already exist")
option(FETCHCONTENT_UPDATES_DISCONNECTED "Enables UPDATE_DISCONNECTED behavior for all content population")
option(FETCHCONTENT_QUIET                "Enables QUIET option for all content population" ON)
set(FETCHCONTENT_BASE_DIR "${CMAKE_BINARY_DIR}/_deps" CACHE PATH "Directory under which to collect all populated content")

# Populate the specified content using details stored from
# an earlier call to FetchContent_Declare().
function(FetchContent_Populate contentName)

  if(NOT contentName)
    message(FATAL_ERROR "Empty contentName not allowed for FetchContent_Populate()")
  endif()

  string(TOLOWER ${contentName} contentNameLower)

  if(ARGN)
    # This is the direct population form with details fully specified
    # as part of the call, so we already have everything we need
    __FetchContent_directPopulate(
      ${contentNameLower}
      SUBBUILD_DIR "${CMAKE_CURRENT_BINARY_DIR}/${contentNameLower}-subbuild"
      SOURCE_DIR   "${CMAKE_CURRENT_BINARY_DIR}/${contentNameLower}-src"
      BINARY_DIR   "${CMAKE_CURRENT_BINARY_DIR}/${contentNameLower}-build"
      ${ARGN}  # Could override any of the above ..._DIR variables
    )

    # Pass source and binary dir variables back to the caller
    set(${contentNameLower}_SOURCE_DIR "${${contentNameLower}_SOURCE_DIR}" PARENT_SCOPE)
    set(${contentNameLower}_BINARY_DIR "${${contentNameLower}_BINARY_DIR}" PARENT_SCOPE)

    # Don't set global properties, or record that we did this population, since
    # this was a direct call outside of the normal declared details form.
    # We only want to save values in the global properties for content that
    # honours the hierarchical details mechanism so that projects are not
    # robbed of the ability to override details set in nested projects.
    return()
  endif()

  # No details provided, so assume they were saved from an earlier call
  # to FetchContent_Declare(). Do a check that we haven't already
  # populated this content before in case the caller forgot to check.
  FetchContent_GetProperties(${contentName})
  if(${contentNameLower}_POPULATED)
    message(FATAL_ERROR "Content ${contentName} already populated in ${${contentNameLower}_SOURCE_DIR}")
  endif()

  string(TOUPPER ${contentName} contentNameUpper)
  set(FETCHCONTENT_SOURCE_DIR_${contentNameUpper}
      "${FETCHCONTENT_SOURCE_DIR_${contentNameUpper}}"
      CACHE PATH "When not empty, overrides where to find pre-populated content for ${contentName}")

  if(FETCHCONTENT_SOURCE_DIR_${contentNameUpper})
    # The source directory has been explicitly provided in the cache,
    # so no population is required
    set(${contentNameLower}_SOURCE_DIR "${FETCHCONTENT_SOURCE_DIR_${contentNameUpper}}")
    set(${contentNameLower}_BINARY_DIR "${FETCHCONTENT_BASE_DIR}/${contentNameLower}-build")

  elseif(FETCHCONTENT_FULLY_DISCONNECTED)
    # Bypass population and assume source is already there from a previous run
    set(${contentNameLower}_SOURCE_DIR "${FETCHCONTENT_BASE_DIR}/${contentNameLower}-src")
    set(${contentNameLower}_BINARY_DIR "${FETCHCONTENT_BASE_DIR}/${contentNameLower}-build")

  else()
    # Support both a global "disconnect all updates" and a per-content
    # update test (either one being set disables updates for this content).
    option(FETCHCONTENT_UPDATES_DISCONNECTED_${contentNameUpper}
           "Enables UPDATE_DISCONNECTED behavior just for population of ${contentName}")
    if(FETCHCONTENT_UPDATES_DISCONNECTED OR
       FETCHCONTENT_UPDATES_DISCONNECTED_${contentNameUpper})
      set(disconnectUpdates True)
    else()
      set(disconnectUpdates False)
    endif()

    if(FETCHCONTENT_QUIET)
      set(quietFlag QUIET)
    else()
      unset(quietFlag)
    endif()

    __FetchContent_getSavedDetails(${contentName} contentDetails)
    if("${contentDetails}" STREQUAL "")
      message(FATAL_ERROR "No details have been set for content: ${contentName}")
    endif()

    __FetchContent_directPopulate(
      ${contentNameLower}
      ${quietFlag}
      UPDATE_DISCONNECTED ${disconnectUpdates}
      SUBBUILD_DIR "${FETCHCONTENT_BASE_DIR}/${contentNameLower}-subbuild"
      SOURCE_DIR   "${FETCHCONTENT_BASE_DIR}/${contentNameLower}-src"
      BINARY_DIR   "${FETCHCONTENT_BASE_DIR}/${contentNameLower}-build"
      # Put the saved details last so they can override any of the
      # the options we set above (this can include SOURCE_DIR or
      # BUILD_DIR)
      ${contentDetails}
    )
  endif()

  __FetchContent_setPopulated(
    ${contentName}
    ${${contentNameLower}_SOURCE_DIR}
    ${${contentNameLower}_BINARY_DIR}
  )

  # Pass variables back to the caller. The variables passed back here
  # must match what FetchContent_GetProperties() sets when it is called
  # with just the content name.
  set(${contentNameLower}_SOURCE_DIR "${${contentNameLower}_SOURCE_DIR}" PARENT_SCOPE)
  set(${contentNameLower}_BINARY_DIR "${${contentNameLower}_BINARY_DIR}" PARENT_SCOPE)
  set(${contentNameLower}_POPULATED  True PARENT_SCOPE)

endfunction()

# Arguments are assumed to be the names of dependencies that have been
# declared previously and should be populated. It is not an error if
# any of them have already been populated (they will just be skipped in
# that case). The command is implemented as a macro so that the variables
# defined by the FetchContent_GetProperties() and FetchContent_Populate()
# calls will be available to the caller.
macro(FetchContent_MakeAvailable)

  foreach(contentName IN ITEMS ${ARGV})
    string(TOLOWER ${contentName} contentNameLower)
    FetchContent_GetProperties(${contentName})
    if(NOT ${contentNameLower}_POPULATED)
      FetchContent_Populate(${contentName})

      # Only try to call add_subdirectory() if the populated content
      # can be treated that way. Protecting the call with the check
      # allows this function to be used for projects that just want
      # to ensure the content exists, such as to provide content at
      # a known location.
      if(EXISTS ${${contentNameLower}_SOURCE_DIR}/CMakeLists.txt)
        add_subdirectory(${${contentNameLower}_SOURCE_DIR}
                         ${${contentNameLower}_BINARY_DIR})
      endif()
    endif()
  endforeach()

endmacro()

else()
  include(FetchContent)
endif()

# Загрузка стандартных модулей
include(GNUInstallDirs)
include(CMakeDependentOption)

# Полезные макросы
include_guard(GLOBAL)

# Создание символических ссылок
macro(myx_create_symlink ORIGINAL_FILENAME LINKNAME)
  if(UNIX AND (NOT EXISTS ${LINKNAME}))
    if(${CMAKE_VERSION} VERSION_LESS 3.14.0)
      execute_process(COMMAND ${CMAKE_COMMAND} -E create_symlink
                              ${ORIGINAL_FILENAME} ${LINKNAME})
    else()
      file(CREATE_LINK ${ORIGINAL_FILENAME} ${LINKNAME} SYMBOLIC)
    endif()
  endif()
endmacro(myx_create_symlink ORIGINAL_FILENAME LINKNAME)

#[=======================================================================[.rst:
myx_find_packages
-----------------

Вспомогательная функция для поиска зависимостей::

  myx_find_packages()

Упрощённый способ поиска необходимых и опциональных зависимостей.
Для поиска зависимостей с учётом особенных требований (например, номер версии)
следует использовать функции `find_package` и `pkg_check_modules`.
Параметр `REQUIRED` содержит перечисление необходимых зависимостей
для поиска с помощью функции `find_package`.
Параметр `OPTIONAL` содержит перечисление опциональных зависимостей
для поиска с помощью функции `find_package`.
Параметр `PKG_REQUIRED` содержит перечисление необходимых зависимостей
для поиска с помощью функции `pkg_check_modules`.
Параметр `PKG_OPTIONAL` содержит перечисление опциональных зависимостей
для поиска с помощью функции `pkg_check_modules`.

#]=======================================================================]

include_guard(GLOBAL)

macro(myx_find_packages)
  set(options)
  set(oneValueArgs)
  set(multiValueArgs REQUIRED OPTIONAL PKG_REQUIRED PKG_OPTIONAL)
  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  foreach(iter ${ARG_REQUIRED})
    find_package(${iter} REQUIRED)
  endforeach()

  foreach(iter ${ARG_OPTIONAL})
    find_package(${iter})
  endforeach()

  if(ARG_PKG_REQUIRED)
    find_package(PkgConfig REQUIRED)
    foreach(iter ${ARG_PKG_REQUIRED})
      string(TOUPPER ${iter} iu)
      pkg_check_modules(${iu} REQUIRED ${iter})
    endforeach()
  endif()

  if(ARG_PKG_OPTIONAL)
    find_package(PkgConfig REQUIRED)
    foreach(iter ${ARG_PKG_OPTIONAL})
      string(TOUPPER ${iter} iu)
      pkg_check_modules(${iu} ${iter})
    endforeach()
  endif()

  foreach(iter IN LISTS oneValueArgs multiValueArgs)
    unset(ARG_${iter})
  endforeach()
  unset(ARG_UNPARSED_ARGUMENTS)
  unset(multiValueArgs)
  unset(oneValueArgs)
  unset(options)
endmacro(myx_find_packages)

include_guard(GLOBAL)

macro(myx_find_qt)
  set(options REQUIRED)
  set(oneValueArgs VERSION)
  set(multiValueArgs COMPONENTS PRIVATE)

  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  if(NOT ARG_VERSION)
    set(ARG_VERSION "5")
    if(QT_DEFAULT_MAJOR_VERSION)
      set(ARG_VERSION "${QT_DEFAULT_MAJOR_VERSION}")
    endif()
  endif()

  if(ARG_REQUIRED)
    set(ARG_REQUIRED "REQUIRED")
  endif()

  if(NOT (ARG_VERSION STREQUAL "5" OR ARG_VERSION STREQUAL "6"))
    message(FATAL_ERROR "Supported Qt versions are 5 and 6")
  endif()

  if(ARG_REQUIRED AND ARG_PRIVATE)
    message("-- MyxCMake: searching required Qt private components: ${ARG_PRIVATE}")
  endif()
  foreach(iter ${ARG_PRIVATE})
    find_package("Qt${ARG_VERSION}${iter}" COMPONENTS Private ${ARG_REQUIRED})
  endforeach()

  if(ARG_REQUIRED AND ARG_COMPONENTS)
    message("-- MyxCMake: searching required Qt components: ${ARG_COMPONENTS}")
  endif()
  foreach(iter ${ARG_COMPONENTS})
    find_package(Qt${ARG_VERSION} COMPONENTS ${iter} ${ARG_REQUIRED})
  endforeach()


  foreach(iter IN LISTS oneValueArgs multiValueArgs)
    unset(ARG_${iter})
  endforeach()
  unset(ARG_UNPARSED_ARGUMENTS)
  unset(multiValueArgs)
  unset(oneValueArgs)
  unset(options)
endmacro()

#[=======================================================================[.rst:
myx_install_relative
--------------------

#]=======================================================================]

macro(myx_install_relative STRIP_DIRECTORY)
  set(options)
  set(oneValueArgs DESTINATION)
  set(multiValueArgs FILES)

  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  foreach(FILE ${ARG_FILES})
    get_filename_component(DIR ${FILE} DIRECTORY)
    string(REPLACE ${STRIP_DIRECTORY} "" RELATIVE_DIR ${DIR})
    INSTALL(FILES ${FILE} DESTINATION ${ARG_DESTINATION}/${RELATIVE_DIR} ${ARG_UNPARSED_ARGUMENTS})
  endforeach()
endmacro(myx_install_relative STRIP_DIRECTORY)

include_guard(GLOBAL)

# Добавление флага к списку используемых после проверки
# возможности его использования текущим компилятором
# Основано на https://github.com/bluescarni/yacma
include(CheckCXXCompilerFlag)

macro(check_enable_cxx_compiler_flag FLAG)
  set(options)
  set(oneValueArgs TARGET)
  set(multiValueArgs)
  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  set(CMAKE_REQUIRED_QUIET TRUE)
  check_cxx_compiler_flag("${FLAG}" check_cxx_flag)
  unset(CMAKE_REQUIRED_QUIET)

  if(check_cxx_flag)
    myx_message_notice("'${FLAG}': flag is supported.")
    if(ARG_TARGET)
      target_compile_options(${ARG_TARGET} PUBLIC ${FLAG})
    else()
      add_compile_options(${FLAG})
    endif()
  else()
    myx_message_status("'${FLAG}': flag is NOT supported.")
  endif()

  unset(check_cxx_flag CACHE)
  foreach(iter IN LISTS oneValueArgs multiValueArgs)
    unset(ARG_${iter})
  endforeach()
  unset(ARG_UNPARSED_ARGUMENTS)
  unset(multiValueArgs)
  unset(oneValueArgs)
  unset(options)
endmacro()


include_guard(GLOBAL)

macro(myx_add_gtest TARGET_NAME)
  set(options)
  set(oneValueArgs)
  set(multiValueArgs)
  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  add_executable(${TARGET_NAME} ${ARG_UNPARSED_ARGUMENTS})

  find_package(GTest)
  if(GTest_FOUND)
    target_link_libraries(${TARGET_NAME} PRIVATE GTest::GTest GTest::Main)
  else()
    if((NOT TARGET gtest) AND (NOT TARGET gtest_main))
      add_subdirectory(/usr/src/googletest/googletest ${CMAKE_BINARY_DIR}/gtest)
    endif()
    target_link_libraries(${TARGET_NAME} PRIVATE gtest gtest_main)
  endif()

  add_test(NAME ${TARGET_NAME} COMMAND ${TARGET_NAME})
  foreach(iter IN LISTS oneValueArgs multiValueArgs)
    unset(ARG_${iter})
  endforeach()
  unset(ARG_UNPARSED_ARGUMENTS)
  unset(multiValueArgs)
  unset(oneValueArgs)
  unset(options)
endmacro()


include_guard(GLOBAL)

macro(myx_add_qtest TARGET_NAME)
  set(options)
  set(oneValueArgs)
  set(multiValueArgs)
  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  find_package(Qt5 COMPONENTS Core Test REQUIRED)
  if(NOT Qt5Test_FOUND)
    myx_message_fatal_error("Qt testing framework is not found")
    return()
  endif()

  foreach(filename ${ARG_UNPARSED_ARGUMENTS})
    get_filename_component(basename ${filename} NAME_WE)
    list(APPEND cpps "${CMAKE_CURRENT_SOURCE_DIR}/${basename}.cpp")
    list(APPEND hpps "${CMAKE_CURRENT_SOURCE_DIR}/${basename}.hpp")
    qt5_wrap_cpp(moc "${basename}.hpp")
    list(APPEND mocs "${moc}")
  endforeach()

  add_executable(${TARGET_NAME} ${mocs} ${cpps} ${hpps})
  target_link_libraries(${TARGET_NAME} PRIVATE Qt5::Core Qt5::Test)

  add_test(NAME ${TARGET_NAME} COMMAND ${TARGET_NAME})

  unset(cpps)
  unset(hpps)
  unset(moc)
  unset(mocs)
  foreach(iter IN LISTS oneValueArgs multiValueArgs)
    unset(ARG_${iter})
  endforeach()
  unset(ARG_UNPARSED_ARGUMENTS)
  unset(multiValueArgs)
  unset(oneValueArgs)
  unset(options)
endmacro()



include(${MYX_CMAKE_LIB_DIR}/ProjectIsTopLevel.cmake)
#[=======================================================================[.rst:

Обёртки для функции `message()`, которые в терминале UNIX
подсвечивают сообщения в зависимости от важности.

#]=======================================================================]

include_guard(GLOBAL)

if(DEFINED ENV{COLORTERM} AND UNIX)
  string(ASCII 27 Esc)
  set(MyxColorReset       "${Esc}[m")
  set(MyxColorBold        "${Esc}[1m")
  set(MyxColorRed         "${Esc}[31m")
  set(MyxColorGreen       "${Esc}[32m")
  set(MyxColorYellow      "${Esc}[33m")
  set(MyxColorBlue        "${Esc}[34m")
  set(MyxColorMagenta     "${Esc}[35m")
  set(MyxColorCyan        "${Esc}[36m")
  set(MyxColorWhite       "${Esc}[37m")
  set(MyxColorBoldRed     "${Esc}[1;31m")
  set(MyxColorBoldGreen   "${Esc}[1;32m")
  set(MyxColorBoldYellow  "${Esc}[1;33m")
  set(MyxColorBoldBlue    "${Esc}[1;34m")
  set(MyxColorBoldMagenta "${Esc}[1;35m")
  set(MyxColorBoldCyan    "${Esc}[1;36m")
  set(MyxColorBoldWhite   "${Esc}[1;37m")
endif()

function(myx_message_fatal_error)
  message(FATAL_ERROR ${MyxColorBoldRed}${ARGV}${MyxColorReset})
endfunction()

function(myx_message_send_error)
  message(SEND_ERROR ${MyxColorBoldRed}${ARGV}${MyxColorReset})
endfunction()

function(myx_message_warning)
  message(WARNING ${MyxColorRed}${ARGV}${MyxColorReset})
endfunction()

function(myx_message_deprecation)
  message(DEPRECATION ${MyxColorBoldMagenta}${ARGV}${MyxColorReset})
endfunction()

function(myx_message_status)
  message(STATUS ${MyxColorMagenta}${ARGV}${MyxColorReset})
endfunction()

function(myx_message_notice)
  message(${MyxColorBold}${ARGV}${MyxColorReset})
endfunction()

function(myx_message)
  message(${MyxColorReset}${ARGV}${MyxColorReset})
endfunction()

include_guard(GLOBAL)

# Создание в каталоге ${CMAKE_BINARY_DIR} стандартных каталогов bin,include,lib,
# а также символических ссылок на каталоги в ${CMAKE_SOURCE_DIR}/files
file(MAKE_DIRECTORY ${CMAKE_BINARY_DIR}/bin)
file(MAKE_DIRECTORY ${CMAKE_BINARY_DIR}/include)
file(MAKE_DIRECTORY ${CMAKE_BINARY_DIR}/lib)
file(MAKE_DIRECTORY ${PROJECT_BINARY_DIR}/bin)
file(MAKE_DIRECTORY ${PROJECT_BINARY_DIR}/include)
file(MAKE_DIRECTORY ${PROJECT_BINARY_DIR}/lib)

myx_create_symlink("${CMAKE_SOURCE_DIR}/files/etc"   "${CMAKE_BINARY_DIR}/etc")
myx_create_symlink("${CMAKE_SOURCE_DIR}/files/log"   "${CMAKE_BINARY_DIR}/log")
myx_create_symlink("${CMAKE_SOURCE_DIR}/files/share" "${CMAKE_BINARY_DIR}/share")
myx_create_symlink("${CMAKE_SOURCE_DIR}/files/var"   "${CMAKE_BINARY_DIR}/var")

include_guard(GLOBAL)

if(NOT MYX_TODAY)
  if(CMAKE_HOST_SYSTEM_NAME STREQUAL Windows)
    execute_process(COMMAND "cmd" " /C date /T" OUTPUT_VARIABLE MYX_TODAY)
  else()
    execute_process(COMMAND "date" "+%d/%m/%Y" OUTPUT_VARIABLE MYX_TODAY)
  endif()
  string(REGEX REPLACE "(..)/(..)/(....).*" "\\3-\\2-\\1" MYX_TODAY ${MYX_TODAY})
endif()

if(NOT MYX_YEAR)
  if(CMAKE_HOST_SYSTEM_NAME STREQUAL Windows)
    execute_process(COMMAND "cmd" " /C date /T" OUTPUT_VARIABLE MYX_YEAR)
  else()
    execute_process(COMMAND "date" "+%d/%m/%Y" OUTPUT_VARIABLE MYX_YEAR)
  endif()
  string(REGEX REPLACE "(..)/(..)/(....).*" "\\3" MYX_YEAR ${MYX_YEAR})
endif()

# Версии CMake, как минимум до 3.8, генерируют некорректные правила для ninja.

include_guard(GLOBAL)

if(${CMAKE_VERSION} VERSION_LESS 3.8 AND CMAKE_GENERATOR MATCHES Ninja)
  myx_message_send_error("Myx: Old CMake versions should use Makefile generator")
endif()

#[=======================================================================[.rst:

Запись результатов сборки проекта внутрь иерархии каталогов с исходными текстами
приводит к засорению файлами формируемыми на этапе сборки, которые затрудняют
разработку, поиск в оригинальных файлах и мешают ориентироваться в проекте.
При работе с несколькими типами сборки, например, отладка и выпуск, появляется
необходимость корректного полного удаления результатов предыдущего варианта.

#]=======================================================================]

include_guard(GLOBAL)

get_filename_component(cmake_source_dir "${CMAKE_SOURCE_DIR}" REALPATH)
get_filename_component(cmake_binary_dir "${CMAKE_BINARY_DIR}" REALPATH)
get_filename_component(project_source_dir "${PROJECT_SOURCE_DIR}" REALPATH)
get_filename_component(project_binary_dir "${PROJECT_BINARY_DIR}" REALPATH)
get_filename_component(cmake_install_prefix "${CMAKE_INSTALL_PREFIX}" REALPATH)

if(cmake_install_prefix STREQUAL cmake_binary_dir)
  myx_message_fatal_error(
    "Myx: Cannot install into build directory ${CMAKE_INSTALL_PREFIX}.")
endif()

if(cmake_install_prefix STREQUAL cmake_source_dir)
  myx_message_fatal_error(
    "Myx: Cannot install into source directory ${CMAKE_INSTALL_PREFIX}.")
endif()

if(cmake_install_prefix STREQUAL project_binary_dir)
  myx_message_fatal_error(
    "Myx: Cannot install into build directory ${CMAKE_INSTALL_PREFIX}.")
endif()

if(cmake_install_prefix STREQUAL project_source_dir)
  myx_message_fatal_error(
    "Myx: Cannot install into source directory ${CMAKE_INSTALL_PREFIX}.")
endif()

if(cmake_binary_dir STREQUAL cmake_source_dir)
  myx_message_fatal_error(
    "Myx: Cannot build in source directory ${CMAKE_SOURCE_DIR}")
endif()

if(project_binary_dir STREQUAL project_source_dir)
  myx_message_fatal_error(
    "Myx: Cannot build in source directory ${CMAKE_SOURCE_DIR}")
endif()

# Очистка от сгенерированных файлов
file(GLOB_RECURSE cmakelists_files RELATIVE ${cmake_source_dir} CMakeLists.txt)
foreach(it ${cmakelists_files})
  get_filename_component(file ${it} REALPATH)
  get_filename_component(dir ${file} DIRECTORY)
  file(REMOVE_RECURSE
    ${dir}/.cmake
    ${dir}/CMakeFiles)
  file(REMOVE
    ${dir}/CMakeFiles
    ${dir}/CMakeCache.txt
    ${dir}/cmake_install.cmake
    ${dir}/compile_commands.json
    ${dir}/Makefile
    ${dir}/.ninja_deps
    ${dir}/.ninja_logs
    ${dir}/build.ninja
    ${dir}/rules.ninja)
endforeach()

unset(cmakelists_files)
unset(cmake_source_dir)
unset(cmake_binary_dir)
unset(project_source_dir)
unset(project_binary_dir)
unset(cmake_install_prefix)

include_guard(GLOBAL)

function(myx_is_semantic_project_version)
  if(NOT ${PROJECT_VERSION_PATCH} MATCHES "([0-9]+)")
    myx_message_error("Myx: Please set project version in X.Y.Z format")
  endif()
endfunction()

function(myx_project_version_int)
  # cmake-format: off
  myx_is_semantic_project_version()
  math(EXPR v "(${PROJECT_VERSION_MAJOR} << 32) + (${PROJECT_VERSION_MINOR} << 16) + ${PROJECT_VERSION_PATCH}")
  set_property(GLOBAL PROPERTY PROJECT_VERSION_INT ${v})
  # cmake-format: on
endfunction()

myx_is_semantic_project_version()

# Если выбран генератор Ninja, то в основном сборочном каталоге создаётся
# файл Makefile, который обрабатывается командой make и передаёт исполнение
# системе сборки ninja. Таким образом можно выполнять команду make,
# даже если правила сборки проекта сгенерированы для ninja.

include_guard(GLOBAL)

if(CMAKE_GENERATOR MATCHES Ninja AND PROJECT_IS_TOP_LEVEL)
  file(WRITE ${CMAKE_BINARY_DIR}/Makefile
    ".PHONY: build\n" "%:\n" "\t@ninja \$@\n" "build:\n" "\t@ninja\n")
endif()

include(${MYX_CMAKE_LIB_DIR}/DownloadContent.cmake)

include_guard(GLOBAL)

if(CMAKE_SYSTEM_NAME STREQUAL Linux)
  set(MYX_CMAKE_LSB_DISTRIBUTOR_ID "unknown")
  set(MYX_CMAKE_LSB_CODENAME "unknown")
  set(MYX_CMAKE_LSB_RELEASE_VERSION "unknown")

  if(CMAKE_CROSSCOMPILING)
    return()
  endif()

  if(EXISTS /etc/mcst_version)
    set(MYX_CMAKE_LSB_DISTRIBUTOR_ID "ElbrusD")
    set(MYX_CMAKE_LSB_CODENAME "Jessie")
    execute_process(COMMAND cat /etc/mcst_version
      OUTPUT_VARIABLE MYX_CMAKE_LSB_RELEASE_VERSION
      OUTPUT_STRIP_TRAILING_WHITESPACE)
  else()
    find_program(LSB_RELEASE_EXECUTABLE lsb_release)
    # cmake-format: off
    if(LSB_RELEASE_EXECUTABLE)
      execute_process(COMMAND ${LSB_RELEASE_EXECUTABLE} -si
        OUTPUT_VARIABLE MYX_CMAKE_LSB_DISTRIBUTOR_ID
        OUTPUT_STRIP_TRAILING_WHITESPACE)
      execute_process(COMMAND ${LSB_RELEASE_EXECUTABLE} -sc
        OUTPUT_VARIABLE MYX_CMAKE_LSB_CODENAME
        OUTPUT_STRIP_TRAILING_WHITESPACE)
      execute_process(COMMAND ${LSB_RELEASE_EXECUTABLE} -sr
        OUTPUT_VARIABLE MYX_CMAKE_LSB_RELEASE_VERSION
        OUTPUT_STRIP_TRAILING_WHITESPACE)
    endif()
    # cmake-format: on
  endif()
endif()

if(CMAKE_CXX_COMPILER_ID STREQUAL "GNU")
  if(CMAKE_COLOR_MAKEFILE)
    check_enable_cxx_compiler_flag(-fdiagnostics-color=auto)
  endif()
endif()

if(CMAKE_CXX_COMPILER_ID STREQUAL "Clang")
  if(CMAKE_COLOR_MAKEFILE)
    check_enable_cxx_compiler_flag(-fcolor-diagnostics)
  endif()
endif()

if((MYX_CMAKE_LSB_DISTRIBUTOR_ID  STREQUAL "AstraLinuxSE") AND
   (MYX_CMAKE_LSB_CODENAME        STREQUAL "smolensk") AND
   (MYX_CMAKE_LSB_RELEASE_VERSION STREQUAL "1.5"))
  find_program(CMAKE_AR NAMES "/usr/bin/x86_64-linux-gnu-gcc-ar-4.7")
  find_program(CMAKE_NM NAMES "/usr/bin/x86_64-linux-gnu-gcc-nm-4.7")
  find_program(CMAKE_RANLIB NAMES "/usr/bin/x86_64-linux-gnu-gcc-ranlib-4.7")

  set(CMAKE_CXX_STANDARD 11)
  set(CMAKE_CXX_EXTENSIONS ON)
  check_enable_cxx_compiler_flag(-Wno-shadow)
endif()

if((MYX_CMAKE_LSB_DISTRIBUTOR_ID  STREQUAL "ElbrusD") AND
   (MYX_CMAKE_LSB_CODENAME        STREQUAL "Jessie") AND
   (MYX_CMAKE_LSB_RELEASE_VERSION VERSION_GREATER "1.4"))
  find_program(CMAKE_AR NAMES "/usr/${CMAKE_SYSTEM_PROCESSOR}-linux/bin/ar")
  find_program(CMAKE_NM NAMES "/usr/${CMAKE_SYSTEM_PROCESSOR}-linux/bin/nm")
  find_program(CMAKE_RANLIB NAMES "/usr/${CMAKE_SYSTEM_PROCESSOR}-linux/bin/ranlib")

  set(CMAKE_CXX_STANDARD 14)
  set(CMAKE_CXX_EXTENSIONS ON)
  check_enable_cxx_compiler_flag(-Wno-invalid-offsetof)
  list(APPEND CMAKE_LIBRARY_PATH "/usr/lib/e2k-linux-gnu")
endif()

#[=======================================================================[.rst:
myx_add_executable
------------------

Вспомогательная функция для создания исполняемого файла::

  myx_add_executable(TARGET_NAME)

Используется для совместимости с версиями CMake раньше 3.11,
в которых было необходимо указать хотя бы один файл с исходными текстами.

#]=======================================================================]

include_guard(GLOBAL)

function(myx_add_executable TARGET_NAME)
  if(${CMAKE_VERSION} VERSION_LESS 3.11.0)
    add_executable(${TARGET_NAME} ${ARGN} "")
  else()
    add_executable(${TARGET_NAME} ${ARGN})
  endif()
endfunction()

#[=======================================================================[.rst:
myx_add_interface_library
-------------------------

Вспомогательная функция для создания интерфейсной библиотеки::

    myx_add_interface_library(TARGET_NAME
      [ PACKAGES packages ] |
      [ LINK_LIBRARIES link_libraries ] |
      [ OUTPUT_NAME output_name ] |
      [ HEADERS headers ])

Обязательные параметры: `TARGET_NAME` - имя библиотеки.
Параметр `OUTPUT_NAME` определяет базовое имя выходных файлов.
Все остальные параметры передаются в стандартную функцию `add_library()`

#]=======================================================================]

include_guard(GLOBAL)

include(CMakePackageConfigHelpers)

if(${CMAKE_VERSION} VERSION_LESS 3.17)
  set(MYX_CMAKE_LIB_DIR_BACKPORT "${CMAKE_CURRENT_LIST_DIR}")
endif()

function(myx_add_interface_library TARGET_NAME)
  if(${CMAKE_VERSION} VERSION_LESS 3.17)
    set(CMAKE_CURRENT_FUNCTION_LIST_DIR ${MYX_CMAKE_LIB_DIR_BACKPORT})
  endif()

  if(TARGET ${TARGET_NAME})
    myx_message_fatal_error("Target ${TARGET_NAME} already exists")
  endif()

  set(options)
  set(oneValueArgs OUTPUT_NAME)
  set(multiValueArgs HEADERS LINK_LIBRARIES PACKAGES)
  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  if(NOT ARG_OUTPUT_NAME)
    set(ARG_OUTPUT_NAME ${TARGET_NAME})
  endif()

  # Вызов стандартной функции `add_library()`
  if(${CMAKE_VERSION} VERSION_LESS 3.19)
    add_library(${TARGET_NAME} INTERFACE)
    target_sources(${TARGET_NAME} INTERFACE $<BUILD_INTERFACE:${ARG_HEADERS}>)
  else()
    add_library(${TARGET_NAME} INTERFACE ${ARG_HEADERS})
  endif()

  foreach(iter ${ARG_PACKAGES})
    target_include_directories(${TARGET_NAME} INTERFACE ${${iter}_INCLUDE_DIRS})
    target_compile_definitions(${TARGET_NAME} INTERFACE ${${iter}_COMPILE_DEFINITIONS})
  endforeach()

  if(ARG_LINK_LIBRARIES)
    foreach(lib ${ARG_LINK_LIBRARIES})
      if(TARGET ${lib})
        set(exlib ${lib})
        get_target_property(type ${lib} TYPE)
        if(type)
          get_target_property(include_dirs ${lib} INTERFACE_INCLUDE_DIRECTORIES)
          if(include_dirs)
            target_include_directories(${TARGET_NAME} INTERFACE ${include_dirs})
          endif()
        endif()
        if(${CMAKE_VERSION} VERSION_GREATER 3.13)
          install(TARGETS ${lib}
                  EXPORT ${TARGET_NAME}Targets
                  COMPONENT DEV)
        else()
          get_target_property(target_type ${lib} TYPE)
          if(target_type STREQUAL "OBJECT_LIBRARY")
            if(TARGET "${lib}_static")
              set(exlib "${lib}_static")
            elseif(TARGET "${lib}_shared")
              set(exlib "${lib}_shared")
            endif()
          endif()
        endif()
        target_link_libraries(${TARGET_NAME} INTERFACE ${exlib})
      endif()
    endforeach()
  endif()

  # Библиотека, состоящая только из заголовочных файлов не требует сборки.
  # Стандартные пути к заголовочным файлам
  target_include_directories(${TARGET_NAME} INTERFACE
    $<BUILD_INTERFACE:${PROJECT_SOURCE_DIR}/include>
    $<INSTALL_INTERFACE:include>
  )

  # Если вызов был выполнен не из проекта верхнего уровня,
  # то созданная цель исключается из цели `all`.
  # При этом сама цель `${TARGET_NAME}` может участвовать в сборке,
  # если окажется в перечне зависимостей.
  if(NOT PROJECT_IS_TOP_LEVEL)
    if(${CMAKE_VERSION} VERSION_GREATER 3.17)
      set_target_properties(${TARGET_NAME} PROPERTIES EXCLUDE_FROM_ALL True)
    else()
      set_target_properties(${TARGET_NAME} PROPERTIES INTERFACE_EXCLUDE_FROM_ALL True)
    endif()
    return()
  endif()

  # Правила для установки библиотек и относящихся к ним файлов.
  # Если вызов был выполнен не из проекта верхнего уровня,
  # то эти правила не обрабатываются.
  write_basic_package_version_file(
    ${CMAKE_CURRENT_BINARY_DIR}/${TARGET_NAME}ConfigVersion.cmake
    VERSION ${PROJECT_VERSION}
    COMPATIBILITY AnyNewerVersion
  )

  configure_package_config_file(
    ${CMAKE_CURRENT_FUNCTION_LIST_DIR}/library-config.cmake.in
    ${CMAKE_CURRENT_BINARY_DIR}/${TARGET_NAME}Config.cmake
    INSTALL_DESTINATION ${CMAKE_INSTALL_LIBDIR}/cmake/${TARGET_NAME}
    NO_SET_AND_CHECK_MACRO
    NO_CHECK_REQUIRED_COMPONENTS_MACRO
  )

  if(${CMAKE_VERSION} VERSION_GREATER 3.13)
    install(
      EXPORT ${TARGET_NAME}Targets
      DESTINATION ${CMAKE_INSTALL_LIBDIR}/cmake/${TARGET_NAME}
      COMPONENT DEV)
  endif()

  install(
    FILES
      ${PROJECT_BINARY_DIR}/${TARGET_NAME}ConfigVersion.cmake
      ${PROJECT_BINARY_DIR}/${TARGET_NAME}Config.cmake
    DESTINATION ${CMAKE_INSTALL_LIBDIR}/cmake/${TARGET_NAME}
    COMPONENT DEV)

  # Формирование файла для pkg-config
  file(WRITE "${PROJECT_BINARY_DIR}/${TARGET_NAME}.pc"
    "prefix=${CMAKE_INSTALL_PREFIX}\n"
    "exec_prefix=${CMAKE_INSTALL_PREFIX}\n"
    "includedir=${CMAKE_INSTALL_PREFIX}/${CMAKE_INSTALL_INCLUDEDIR}\n"
    "\n"
    "Name: ${TARGET_NAME}\n"
    "Description: ${TARGET_NAME} library\n"
    "Version: ${PROJECT_VERSION}\n"
    "\n"
    "Requires:\n"
    "Cflags: -I${CMAKE_INSTALL_PREFIX}/${CMAKE_INSTALL_INCLUDEDIR}\n")

  # Установка файла для pkg-config
  install(
    FILES "${PROJECT_BINARY_DIR}/${TARGET_NAME}.pc"
    DESTINATION "${CMAKE_INSTALL_LIBDIR}/pkgconfig"
    COMPONENT DEV)

  # Установка библиотеки из заголовочных файлов
  target_include_directories(${TARGET_NAME} SYSTEM INTERFACE $<INSTALL_INTERFACE:${CMAKE_INSTALL_INCLUDEDIR}>)
  if(${CMAKE_VERSION} VERSION_GREATER 3.13)
    install(
      TARGETS ${TARGET_NAME}
      EXPORT ${TARGET_NAME}Targets
      COMPONENT DEV)
  endif()

  # Установка публичных заголовочных файлов
  if(PROJECT_IS_TOP_LEVEL AND ARG_HEADERS)
    myx_install_relative(${PROJECT_SOURCE_DIR}
      FILES ${ARG_HEADERS}
      DESTINATION ${CMAKE_INSTALL_PREFIX}
      COMPONENT DEV)
  endif()
endfunction()

#[=======================================================================[.rst:
myx_add_object_library
----------------------

Вспомогательная функция для создания объектной библиотеки::

    myx_add_object_library(TARGET_NAME
      [ OUTPUT_NAME output_name ] |
      [ NO_EXPORT ] |
      [ EXPORT_FILE_NAME file_name ] |
      [ EXPORT_BASE_NAME base_name ])

Обязательные параметры: `TARGET_NAME` - имя библиотеки.
Параметр `OUTPUT_NAME` определяет базовое имя выходных файлов.
Если указана опция `NO_EXPORT`, то файл экспорта не генерируется.
Параметр `EXPORT_FILE_NAME` задаёт имя заголовочного файла экспортируемых
переменных, а `EXPORT_BASE_NAME` - базовый суффикс для формирования имён переменных.
Все остальные параметры передаются в стандартную функцию `add_library()`

#]=======================================================================]

include_guard(GLOBAL)

if(${CMAKE_VERSION} VERSION_LESS 3.17)
  set(MYX_CMAKE_LIB_DIR_BACKPORT "${CMAKE_CURRENT_LIST_DIR}")
endif()

function(myx_add_object_library TARGET_NAME)
  if(${CMAKE_VERSION} VERSION_LESS 3.17)
    set(CMAKE_CURRENT_FUNCTION_LIST_DIR ${MYX_CMAKE_LIB_DIR_BACKPORT})
  endif()

  include(CMakePackageConfigHelpers)

  set(options NO_EXPORT)
  set(oneValueArgs OUTPUT_NAME EXPORT_FILE_NAME EXPORT_BASE_NAME)
  set(multiValueArgs)
  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  if(NOT ARG_OUTPUT_NAME)
    set(ARG_OUTPUT_NAME ${TARGET_NAME})
  endif()

  # Вызов стандартной функции `add_library()`
  add_library(${TARGET_NAME} OBJECT ${ARG_UNPARSED_ARGUMENTS})

  string(TOUPPER ${TARGET_NAME} project_name_upper)
  # Опция для разрешения сборки динамической библиотеки
  cmake_dependent_option(${project_name_upper}_BUILD_SHARED
    "Build shared library for ${TARGET_NAME}" ON "BUILD_SHARED_LIBS" OFF)
  # Опция для разрешения сборки статической библиотеки
  cmake_dependent_option(${project_name_upper}_BUILD_STATIC
    "Build static library for ${TARGET_NAME}" ON "NOT BUILD_SHARED_LIBS" OFF)

  # Стандартные пути к заголовочным файлам
  target_include_directories(${TARGET_NAME}
    PUBLIC
      $<BUILD_INTERFACE:${PROJECT_SOURCE_DIR}/include>
    PRIVATE
      $<BUILD_INTERFACE:${PROJECT_SOURCE_DIR}/src>
      $<INSTALL_INTERFACE:${CMAKE_INSTALL_INCLUDEDIR}>
  )

  # Сборка позиционно-независимых объектных файлов нужна
  # для создания динамической библиотеки
  set_target_properties(${TARGET_NAME} PROPERTIES POSITION_INDEPENDENT_CODE ON)

  if(NOT ARG_NO_EXPORT)
    include(GenerateExportHeader)
    if(NOT EXPORT_BASE_NAME)
      set(ARG_EXPORT_BASE_NAME ${project_name_upper})
    endif()

    if(NOT ARG_EXPORT_FILE_NAME)
      set(ARG_EXPORT_FILE_NAME "${PROJECT_SOURCE_DIR}/include/${TARGET_NAME}/${TARGET_NAME}_export.hpp")
    endif()

    generate_export_header(${TARGET_NAME}
      BASE_NAME ${ARG_EXPORT_BASE_NAME}
      EXPORT_MACRO_NAME "EXPORT_${ARG_EXPORT_BASE_NAME}"
      DEPRECATED_MACRO_NAME "DEPRECATED_${ARG_EXPORT_BASE_NAME}"
      NO_DEPRECATED_MACRO_NAME "NO_DEPRECATED_${ARG_EXPORT_BASE_NAME}"
      NO_EXPORT_MACRO_NAME "NO_EXPORT_${ARG_EXPORT_BASE_NAME}"
      STATIC_DEFINE "STATIC_DEFINE_${ARG_EXPORT_BASE_NAME}"
      EXPORT_FILE_NAME ${ARG_EXPORT_FILE_NAME}
      DEFINE_NO_DEPRECATED
    )
    set_property(TARGET ${TARGET_NAME} APPEND PROPERTY INTERFACE_HEADER_FILES "${ARG_EXPORT_FILE_NAME}")
  endif()

  # Цель для создания динамической библиотеки из объектных файлов
  if(${project_name_upper}_BUILD_SHARED)
    # Для создания динамической библиотеки используются объектные файлы цели ${TARGET_NAME}
    add_library(${TARGET_NAME}_shared SHARED $<TARGET_OBJECTS:${TARGET_NAME}>)
    # Установка дополнительных свойств для цели ${TARGET_NAME}_shared
    # VERSION:     версия библиотеки
    # SOVERSION:   мажорная версия библиотеки
    # LIBRARY_OUTPUT_DIRECTORY: каталог для записи результатов сборки
    # OUTPUT_NAME: базовое имя выходного файла (без учёта расширения)
    set_target_properties(${TARGET_NAME}_shared PROPERTIES
      VERSION ${PROJECT_VERSION}
      SOVERSION ${PROJECT_VERSION_MAJOR}
      LIBRARY_OUTPUT_DIRECTORY ${CMAKE_BINARY_DIR}/${CMAKE_INSTALL_LIBDIR}
      OUTPUT_NAME ${ARG_OUTPUT_NAME})
  endif()

  # Цель для создания статической библиотеки из объектных файлов
  if(${project_name_upper}_BUILD_STATIC)
    # Для создания статической библиотеки используются
    # объектные файлы цели ${TARGET_NAME}
    add_library(${TARGET_NAME}_static STATIC $<TARGET_OBJECTS:${TARGET_NAME}>)
    # Установка дополнительных свойств для цели ${TARGET_NAME}_static
    # ARCHIVE_OUTPUT_DIRECTORY: каталог для записи результатов сборки
    # OUTPUT_NAME: базовое имя выходного файла (без учёта расширения)
    set_target_properties(${TARGET_NAME}_static PROPERTIES
      ARCHIVE_OUTPUT_DIRECTORY ${CMAKE_BINARY_DIR}/${CMAKE_INSTALL_LIBDIR}
      OUTPUT_NAME ${ARG_OUTPUT_NAME})
  endif()

  # Если вызов был выполнен не из проекта верхнего уровня,
  # то созданная цель исключается из цели `all`.
  # При этом сама цель `${TARGET_NAME}` может участвовать в сборке,
  # если окажется в перечне зависимостей.
  if(NOT PROJECT_IS_TOP_LEVEL)
    set_target_properties(${TARGET_NAME} PROPERTIES EXCLUDE_FROM_ALL True)
    return()
  endif()

  # Правила для установки библиотек и относящихся к ним файлов.
  # Если вызов был выполнен не из проекта верхнего уровня,
  # то эти правила не обрабатываются.
  write_basic_package_version_file(
    ${CMAKE_CURRENT_BINARY_DIR}/${TARGET_NAME}ConfigVersion.cmake
    VERSION ${PROJECT_VERSION}
    COMPATIBILITY AnyNewerVersion
  )

  configure_package_config_file(
    ${CMAKE_CURRENT_FUNCTION_LIST_DIR}/library-config.cmake.in
    ${CMAKE_CURRENT_BINARY_DIR}/${TARGET_NAME}Config.cmake
    INSTALL_DESTINATION ${CMAKE_INSTALL_LIBDIR}/cmake/${TARGET_NAME}
    NO_SET_AND_CHECK_MACRO
    NO_CHECK_REQUIRED_COMPONENTS_MACRO
  )

  install(EXPORT ${TARGET_NAME}Targets
    DESTINATION ${CMAKE_INSTALL_LIBDIR}/cmake/${TARGET_NAME}
    COMPONENT DEV
  )

  install(
    FILES
      ${PROJECT_BINARY_DIR}/${TARGET_NAME}ConfigVersion.cmake
      ${PROJECT_BINARY_DIR}/${TARGET_NAME}Config.cmake
    DESTINATION ${CMAKE_INSTALL_LIBDIR}/cmake/${TARGET_NAME}
    COMPONENT DEV
  )

  # Формирование файла для pkg-config
  file(WRITE "${PROJECT_BINARY_DIR}/${TARGET_NAME}.pc"
    "prefix=${CMAKE_INSTALL_PREFIX}\n"
    "exec_prefix=${CMAKE_INSTALL_PREFIX}\n"
    "includedir=${CMAKE_INSTALL_PREFIX}/${CMAKE_INSTALL_INCLUDEDIR}\n"
    "libdir=${CMAKE_INSTALL_PREFIX}/${CMAKE_INSTALL_LIBDIR}\n"
    "\n"
    "Name: ${TARGET_NAME}\n"
    "Description: ${TARGET_NAME} library\n"
    "Version: ${PROJECT_VERSION}\n"
    "\n"
    "Requires:\n"
    "Cflags: -I${CMAKE_INSTALL_PREFIX}/${CMAKE_INSTALL_INCLUDEDIR}\n"
    "Libs: -L${CMAKE_INSTALL_PREFIX}/${CMAKE_INSTALL_LIBDIR} -l${TARGET_NAME}\n")

  # Установка файла для pkg-config
  install(
    FILES "${PROJECT_BINARY_DIR}/${TARGET_NAME}.pc"
    COMPONENT DEV
    DESTINATION "${CMAKE_INSTALL_LIBDIR}/pkgconfig")

  # Установка динамической библиотеки
  if(TARGET ${TARGET_NAME}_shared)
    install(
      TARGETS ${TARGET_NAME}_shared
      EXPORT ${TARGET_NAME}Targets
      LIBRARY DESTINATION ${CMAKE_INSTALL_LIBDIR}
      COMPONENT LIB
    )
  endif()

  # Установка статической библиотеки
  if(TARGET ${TARGET_NAME}_static)
    install(
      TARGETS ${TARGET_NAME}_static
      EXPORT ${TARGET_NAME}Targets
      ARCHIVE DESTINATION ${CMAKE_INSTALL_LIBDIR}
      COMPONENT STATIC
    )
  endif()
endfunction()

include_guard(GLOBAL)

function(myx_target_setup TARGET_NAME)
  set(options)
  set(oneValueArgs PCH)
  set(multiValueArgs PACKAGES
                     COMPILE_DEFINITIONS INCLUDE_DIRECTORIES LINK_LIBRARIES
                     CPP PUBLIC_HEADERS PRIVATE_HEADERS)
  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  if(NOT TARGET ${TARGET_NAME})
    myx_message_fatal_error("Target ${TARGET_NAME} does not exists.")
  endif()

  get_target_property(target_type ${TARGET_NAME} TYPE)
  if(${target_type} STREQUAL "INTERFACE_LIBRARY")
    myx_message_fatal_error("This function will not work for interface targets.")
  endif()

  foreach(iter ${ARG_PACKAGES})
    target_include_directories(${TARGET_NAME} PRIVATE ${${iter}_INCLUDE_DIRS})
    target_compile_definitions(${TARGET_NAME} PRIVATE ${${iter}_COMPILE_DEFINITIONS})
  endforeach()

  if(ARG_CPP)
    set_property(TARGET ${TARGET_NAME} APPEND PROPERTY CPP_FILES ${ARG_CPP})
    set_property(TARGET ${TARGET_NAME} APPEND PROPERTY TR_FILES ${ARG_CPP})
  endif()

  if(ARG_PUBLIC_HEADERS)
    set_property(TARGET ${TARGET_NAME} APPEND PROPERTY PUBLIC_HEADER_FILES "${ARG_PUBLIC_HEADERS}")
    set_property(TARGET ${TARGET_NAME} APPEND PROPERTY TR_FILES ${ARG_PUBLIC_HEADERS})
  endif()

  if(ARG_PRIVATE_HEADERS)
    set_property(TARGET ${TARGET_NAME} APPEND PROPERTY PRIVATE_HEADER_FILES "${ARG_PRIVATE_HEADERS}")
    set_property(TARGET ${TARGET_NAME} APPEND PROPERTY TR_FILES ${ARG_PRIVATE_HEADERS})
  endif()

  if(ARG_PCH)
    if((${CMAKE_VERSION} VERSION_GREATER_EQUAL 3.16) AND (PROJECT_IS_TOP_LEVEL OR MYX_USE_LOCAL_DIRECTORIES))
      target_precompile_headers(${TARGET_NAME} PRIVATE ${ARG_PCH})
    else()
      target_compile_options(${TARGET_NAME} PRIVATE -include ${ARG_PCH})
    endif()
    set_property(TARGET ${TARGET_NAME} APPEND PROPERTY PRIVATE_HEADER_FILES "${ARG_PCH}")
  endif()

  target_include_directories(${PROJECT_NAME} PRIVATE
    $<BUILD_INTERFACE:${PROJECT_BINARY_DIR}/include>)

  if(IS_DIRECTORY "${PROJECT_SOURCE_DIR}/src")
    target_include_directories(${PROJECT_NAME} PRIVATE
      $<BUILD_INTERFACE:${PROJECT_SOURCE_DIR}/src>)
  endif()

  if(target_type STREQUAL "EXECUTABLE")
    if(IS_DIRECTORY "${PROJECT_SOURCE_DIR}/include")
      target_include_directories(${PROJECT_NAME} PRIVATE
        $<BUILD_INTERFACE:${PROJECT_SOURCE_DIR}/include>)
    endif()
    if(IS_DIRECTORY "${PROJECT_SOURCE_DIR}/include/${PROJECT_NAME}")
      target_include_directories(${PROJECT_NAME} PRIVATE
        $<BUILD_INTERFACE:${PROJECT_SOURCE_DIR}/include/${PROJECT_NAME}>)
    endif()

    set_target_properties(${TARGET_NAME} PROPERTIES
      POSITION_INDEPENDENT_CODE ON
      RUNTIME_OUTPUT_DIRECTORY ${CMAKE_BINARY_DIR}/${CMAKE_INSTALL_BINDIR}
    )

    if(CMAKE_CXX_COMPILE_OPTIONS_PIE)
      target_compile_options(${TARGET_NAME} PUBLIC ${CMAKE_CXX_COMPILE_OPTIONS_PIE})
    endif()
    install(TARGETS ${TARGET_NAME} COMPONENT MAIN RUNTIME DESTINATION ${CMAKE_INSTALL_BINDIR})
  endif()

  if(${CMAKE_VERSION} VERSION_LESS "3.11.99")
    target_sources(${TARGET_NAME} PRIVATE $<BUILD_INTERFACE:${ARG_PUBLIC_HEADERS}>)
    target_sources(${TARGET_NAME} PRIVATE $<BUILD_INTERFACE:${ARG_CPP} ${ARG_PCH} ${ARG_PRIVATE_HEADERS}>)
  else()
    target_sources(${TARGET_NAME} PRIVATE ${ARG_PUBLIC_HEADERS})
    target_sources(${TARGET_NAME} PRIVATE ${ARG_CPP} ${ARG_PCH} ${ARG_PRIVATE_HEADERS})
  endif()
  target_compile_definitions(${TARGET_NAME} PRIVATE ${ARG_COMPILE_DEFINITIONS})
  target_include_directories(${TARGET_NAME} PRIVATE ${ARG_INCLUDE_DIRECTORIES})

  # CMake до версии 3.12 не умеет извлекать из целей типа `OBJECT_LIBRARY`
  # информацию о заголовочных файлах. Это обход.
  if(${CMAKE_VERSION} VERSION_GREATER "3.11.99")
    target_link_libraries(${TARGET_NAME} PRIVATE ${ARG_LINK_LIBRARIES})
  else()
    if((NOT ${target_type} STREQUAL "OBJECT_LIBRARY") AND
       (NOT ${target_type} STREQUAL "EXECUTABLE"))
      target_link_libraries(${TARGET_NAME} PRIVATE ${ARG_LINK_LIBRARIES})
    else()
      foreach(link_library ${ARG_LINK_LIBRARIES})
        if(TARGET ${link_library})
          get_target_property(library_type ${link_library} TYPE)
          if(library_type)
            get_target_property(include_dirs ${link_library} INTERFACE_INCLUDE_DIRECTORIES)
            if(include_dirs)
              target_include_directories(${TARGET_NAME} PUBLIC ${include_dirs})
            endif()
            if(${target_type} STREQUAL "EXECUTABLE")
              if(${library_type} STREQUAL "OBJECT_LIBRARY")
                if(TARGET ${link_library}_static)
                  target_link_libraries(${TARGET_NAME} PRIVATE ${link_library}_static)
                else()
                  target_link_libraries(${TARGET_NAME} PRIVATE ${link_library}_shared)
                endif()
              else()
                target_link_libraries(${TARGET_NAME} PRIVATE ${link_library})
              endif()
            endif()
          endif()
        else()
          if((${CMAKE_VERSION} VERSION_GREATER "3.7.99") OR (NOT target_type STREQUAL "OBJECT_LIBRARY"))
            target_link_libraries(${TARGET_NAME} PRIVATE ${link_library})
          endif()
        endif()
      endforeach()
    endif()
  endif()

  # Установка публичных заголовочных файлов
  if(PROJECT_IS_TOP_LEVEL AND ARG_PUBLIC_HEADERS)
    myx_install_relative(${PROJECT_SOURCE_DIR}
      FILES ${ARG_PUBLIC_HEADERS}
      DESTINATION ${CMAKE_INSTALL_PREFIX}
      COMPONENT DEV
    )
  endif()
endfunction()

include_guard(GLOBAL)

function(myx_target_setup_qt TARGET_NAME)
  set(options)
  set(oneValueArgs)
  set(multiValueArgs COMPONENTS PRIVATE PUBLIC_MOC PRIVATE_MOC UI QRC LANGS)

  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  set(myx_qt_ver "5")
  if(QT_DEFAULT_MAJOR_VERSION)
    set(myx_qt_ver "${QT_DEFAULT_MAJOR_VERSION}")
  endif()

  if(NOT (myx_qt_ver STREQUAL "5" OR myx_qt_ver STREQUAL "6"))
    myx_message_fatal_error("Supported Qt versions are 5 and 6")
  endif()

  get_target_property(target_type ${TARGET_NAME} TYPE)

  foreach(iter ${ARG_COMPONENTS})
    if(target_type STREQUAL "INTERFACE_LIBRARY")
      target_include_directories(${TARGET_NAME} INTERFACE ${Qt${myx_qt_ver}${iter}_INCLUDE_DIRS})
    else()
      target_include_directories(${TARGET_NAME} PRIVATE ${Qt${myx_qt_ver}${iter}_INCLUDE_DIRS})
      if(NOT iter STREQUAL "LinguistTools")
        if(target_type STREQUAL "EXECUTABLE")
          target_link_libraries(${TARGET_NAME} PRIVATE Qt${myx_qt_ver}::${iter})
        endif()
        if(target_type STREQUAL "SHARED_LIBRARY")
          target_link_libraries(${TARGET_NAME} PUBLIC Qt${myx_qt_ver}::${iter})
        endif()
        if((${CMAKE_VERSION} VERSION_GREATER 3.8.0) AND (target_type STREQUAL "OBJECT_LIBRARY"))
          target_link_libraries(${TARGET_NAME} PUBLIC Qt${myx_qt_ver}::${iter})
        else()
          target_include_directories(${TARGET_NAME} PUBLIC ${Qt${myx_qt_ver}${iter}_INCLUDE_DIRS})
        endif()
      endif()
    endif()
  endforeach()

  foreach(iter ${ARG_PRIVATE})
    if(target_type STREQUAL "INTERFACE_LIBRARY")
      target_include_directories(${TARGET_NAME} INTERFACE ${Qt${myx_qt_ver}${iter}_PRIVATE_INCLUDE_DIRS})
    else()
      target_include_directories(${TARGET_NAME} PRIVATE ${Qt${myx_qt_ver}${iter}_PRIVATE_INCLUDE_DIRS})
    endif()
  endforeach()

  if(target_type STREQUAL "EXECUTABLE")
    target_compile_options(${TARGET_NAME} PRIVATE ${Qt${myx_qt_ver}Core_EXECUTABLE_COMPILE_FLAGS})
  endif()

  if(ARG_PUBLIC_MOC)
    set_property(TARGET ${TARGET_NAME} APPEND PROPERTY PUBLIC_HEADER_FILES "${ARG_PUBLIC_MOC}")
    set_property(TARGET ${TARGET_NAME} APPEND PROPERTY TR_FILES ${ARG_PUBLIC_MOC})
  endif()

  if(ARG_PRIVATE_MOC)
    set_property(TARGET ${TARGET_NAME} APPEND PROPERTY PRIVATE_HEADER_FILES "${ARG_PRIVATE_MOC}")
    set_property(TARGET ${TARGET_NAME} APPEND PROPERTY TR_FILES ${ARG_PRIVATE_MOC})
  endif()

  if(ARG_PUBLIC_MOC OR ARG_PRIVATE_MOC)
    if(myx_qt_ver EQUAL 5)
      qt5_wrap_cpp(moc_cpp ${ARG_PUBLIC_MOC} ${ARG_PRIVATE_MOC})
    elseif(myx_qt_ver EQUAL 6)
      qt6_wrap_cpp(moc_cpp ${ARG_PUBLIC_MOC} ${ARG_PRIVATE_MOC})
    endif()
  endif()

  if(ARG_QRC)
    if(myx_qt_ver EQUAL 5)
      qt5_add_resources(qrc_cpp ${ARG_QRC})
    elseif(myx_qt_ver EQUAL 6)
      qt6_add_resources(qrc_cpp ${ARG_QRC})
    endif()
  endif()

  if(ARG_UI)
    if(NOT COMMAND qt${myx_qt_ver}_wrap_ui)
      message(WARNING "MyxCMake: Widgets is required to process UI")
    else()
      if(myx_qt_ver EQUAL 5)
        qt5_wrap_ui(ui_h ${ARG_UI})
      elseif(myx_qt_ver EQUAL 6)
        qt6_wrap_ui(ui_h ${ARG_UI})
      endif()
      set_property(TARGET ${TARGET_NAME} APPEND PROPERTY TR_FILES ${ARG_UI})
      target_link_libraries(${TARGET_NAME} PRIVATE Qt${myx_qt_ver}::Widgets)
      # TODO
      target_include_directories(${PROJECT_NAME} PRIVATE ${PROJECT_BINARY_DIR})
    endif()
  endif()

  # Перечень файлов, подлежащих переводу
  if(target_type STREQUAL "INTERFACE_LIBRARY")
    get_target_property(tr ${TARGET_NAME} INTERFACE_TR_FILES)
  else()
    get_target_property(tr ${TARGET_NAME} TR_FILES)
  endif()

  # Формирование файла ресурсов с переводами
  if(tr AND ARG_LANGS)
    if(NOT COMMAND qt${myx_qt_ver}_create_translation)
      message(WARNING "MyxCMake: LinguistTools is required to process LANGS")
    else()
      # Заглавие файла ресурсов
      file(WRITE ${PROJECT_BINARY_DIR}/${TARGET_NAME}_l10n.qrc "<RCC><qresource prefix=\"/l10n\">\n")
      # Для каждого языка, указанное в параметре LANGS
      foreach(iter ${ARG_LANGS})
        # Создание или обновление файла переводов в каталоге ${PROJECT_SOURCE_DIR}/l10n
        # и его компиляция в каталог ${PROJECT_BINARY_DIR}
        if(myx_qt_ver EQUAL 5)
          qt5_create_translation(qm ${tr}
            "${PROJECT_SOURCE_DIR}/l10n/${TARGET_NAME}_${iter}.ts"
            OPTIONS -I ${PROJECT_SOURCE_DIR}/include -I ${PROJECT_SOURCE_DIR}/include/${PROJECT_NAME})
        elseif(myx_qt_ver EQUAL 6)
          qt6_create_translation(qm ${tr}
            "${PROJECT_SOURCE_DIR}/l10n/${TARGET_NAME}_${iter}.ts"
            OPTIONS -I ${PROJECT_SOURCE_DIR}/include -I ${PROJECT_SOURCE_DIR}/include/${PROJECT_NAME})
        endif()
        # Добавление записи для скомпилированного файла переводов в ресурсный файл
        file(APPEND ${PROJECT_BINARY_DIR}/${TARGET_NAME}_l10n.qrc
          "<file alias=\"${TARGET_NAME}_${iter}.qm\">${TARGET_NAME}_${iter}.qm</file>\n")
        # Добавление скомпилированного файла переводов к списку зависимостей для цели
        target_sources(${TARGET_NAME} PRIVATE ${qm})
      endforeach()
      # Окончание файла ресурсов
      file(APPEND ${PROJECT_BINARY_DIR}/${TARGET_NAME}_l10n.qrc "</qresource></RCC>\n")
      # Компиляция файла ресурсов с переводами
      if(myx_qt_ver EQUAL 5)
        qt5_add_resources(qrc_l10n ${PROJECT_BINARY_DIR}/${TARGET_NAME}_l10n.qrc)
      elseif(myx_qt_ver EQUAL 6)
        qt6_add_resources(qrc_l10n ${PROJECT_BINARY_DIR}/${TARGET_NAME}_l10n.qrc)
      endif()
      target_sources(${TARGET_NAME} PRIVATE ${qrc_l10n})
    endif()
  endif()
  unset(tr)
  if(target_type STREQUAL "INTERFACE_LIBRARY")
    target_sources(${TARGET_NAME} INTERFACE ${ARG_PUBLIC_MOC} ${ARG_PRIVATE_MOC} ${moc_cpp} ${ui_h} ${qrc_cpp})
  else()
    target_sources(${TARGET_NAME} PRIVATE ${ARG_PUBLIC_MOC} ${ARG_PRIVATE_MOC} ${moc_cpp} ${ui_h} ${qrc_cpp})
  endif()

  # Установка публичных заголовочных файлов
  if(PROJECT_IS_TOP_LEVEL)
    myx_install_relative(${PROJECT_SOURCE_DIR}
      FILES ${ARG_PUBLIC_MOC}
      DESTINATION ${CMAKE_INSTALL_PREFIX}
      COMPONENT DEV
    )
  endif()
endfunction()


if(${CMAKE_VERSION} VERSION_LESS 3.17.0)
  set(MYX_CMAKE_LIB_UNCRUSTIFY_DIR_BACKPORT "${CMAKE_CURRENT_LIST_DIR}")
endif()

find_program(UNCRUSTIFY_EXE NAMES uncrustify)

function(myx_uncrustify TARGET_NAME)
  if(${CMAKE_VERSION} VERSION_LESS 3.17.0)
    set(CMAKE_CURRENT_FUNCTION_LIST_DIR ${MYX_CMAKE_LIB_UNCRUSTIFY_DIR_BACKPORT})
  endif()

  set(options)
  set(oneValueArgs CONFIG)
  set(multiValueArgs)
  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  if(NOT ARG_CONFIG)
    set(ARG_CONFIG "${PROJECT_SOURCE_DIR}/uncrustify.cfg")
  endif()

  if(NOT EXISTS ${ARG_CONFIG})
    myx_message_notice("-- MyxCMake: uncrustify config is not found")
    return()
  endif()

  if(NOT UNCRUSTIFY_EXE)
    myx_message_notice("-- MyxCMake: uncrustify executable is not found")
    return()
  endif()

  get_filename_component(CONFIG_DIR ${ARG_CONFIG} DIRECTORY)
  if(NOT "${CMAKE_SOURCE_DIR}" STREQUAL "${CONFIG_DIR}")
    myx_message("-- MyxCMake: skip uncrustify for project ${PROJECT_NAME}")
    return()
  endif()

  if(NOT TARGET myx-uncrustify)
    add_custom_target(myx-uncrustify)
  endif()
  if(NOT TARGET myx-uncrustify-check)
    add_custom_target(myx-uncrustify-check)
  endif()
  if(NOT TARGET myx-uncrustify-append-comments)
    add_custom_target(myx-uncrustify-append-comments)
  endif()

  # Динамически сгенерированные файлы исключаются
  get_target_property(target_type ${TARGET_NAME} TYPE)
  if((${target_type} STREQUAL "INTERFACE_LIBRARY") AND (${CMAKE_VERSION} VERSION_LESS 3.17.0))
    get_target_property(target_sources ${TARGET_NAME} INTERFACE_SOURCES)
    if(target_sources)
      list(APPEND all_sources ${target_sources})
    endif()
  else()
    get_target_property(target_sources ${TARGET_NAME} SOURCES)
    if(target_sources)
      list(APPEND all_sources ${target_sources})
    endif()
  endif()

  foreach(iter ${all_sources})
    string(FIND ${iter} ${CMAKE_BINARY_DIR} pos)
    if(pos EQUAL -1)
      list(APPEND sources ${iter})
    endif()
  endforeach()

  set(fixed_config ${PROJECT_BINARY_DIR}/uncrustify-${TARGET_NAME}.cfg)
  add_custom_command(OUTPUT ${fixed_config}
                     DEPENDS ${ARG_CONFIG}
                     COMMAND ${UNCRUSTIFY_EXE} --update-config-with-doc
                                               -c ${ARG_CONFIG} -o ${fixed_config})
  list(APPEND options -c ${fixed_config})
  # cmake-format: off
  add_custom_target(${TARGET_NAME}-uncrustify-check
    DEPENDS ${fixed_config}
    COMMAND ${UNCRUSTIFY_EXE} ${options} --check ${sources})

  list(APPEND options --replace --no-backup)
  add_custom_target(${TARGET_NAME}-uncrustify
    DEPENDS ${fixed_config}
    COMMAND ${UNCRUSTIFY_EXE} ${options} --mtime ${sources})

  add_custom_target(${TARGET_NAME}-uncrustify-append-comments
    DEPENDS ${fixed_config}
    COMMAND ${UNCRUSTIFY_EXE} ${options}
            --set cmt_insert_class_header=${CMAKE_CURRENT_FUNCTION_LIST_DIR}/classheader.txt
            --set cmt_insert_file_footer=${CMAKE_CURRENT_FUNCTION_LIST_DIR}/filefooter.txt
            --set cmt_insert_file_header=${CMAKE_CURRENT_FUNCTION_LIST_DIR}/fileheader.txt
            --set cmt_insert_func_header=${CMAKE_CURRENT_FUNCTION_LIST_DIR}/funcheader.txt
            --set cmt_insert_before_ctor_dtor=true --mtime ${sources})
  # cmake-format: on

  add_dependencies(myx-uncrustify ${TARGET_NAME}-uncrustify)
  add_dependencies(myx-uncrustify-check ${TARGET_NAME}-uncrustify-check)
  add_dependencies(myx-uncrustify-append-comments ${TARGET_NAME}-uncrustify-append-comments)
endfunction()

if(${CMAKE_VERSION} VERSION_LESS 3.17)
  set(MYX_CMAKE_LIB_DOC_DIR_BACKPORT "${CMAKE_CURRENT_LIST_DIR}")
endif()

find_package(Doxygen QUIET)

function(myx_doc_doxygen TARGET_NAME)
  if(${CMAKE_VERSION} VERSION_LESS 3.17)
    set(CMAKE_CURRENT_FUNCTION_LIST_DIR ${MYX_CMAKE_LIB_DOC_DIR_BACKPORT})
  endif()

  if(NOT ${PROJECT_BINARY_DIR} STREQUAL ${CMAKE_BINARY_DIR})
    return()
  endif()

  if(DOXYGEN_FOUND)
    set(DOXYGEN_FOUND ON CACHE STRING "Doxygen documentation generator enabled" FORCE)
    set(DOXYGEN_EXECUTABLE "${DOXYGEN_EXECUTABLE}" CACHE STRING "Path to Doxygen executable")
  else()
    set(DOXYGEN_FOUND OFF CACHE STRING "Doxygen documentation generator disabled" FORCE)
  endif()

  set(target myx-doc-doxygen-${TARGET_NAME})
  if(TARGET ${target})
    myx_message_warning("Target ${target} already defined")
    return()
  endif()

  if(NOT DOXYGEN_FOUND)
    add_custom_target(${target} VERBATIM COMMENT "Doxygen is not found. Skipping target ${target} build")
    return()
  endif()

  set(options)
  set(oneValueArgs HTML LATEX XML LANGUAGE DOXYFILE_TEMPLATE)
  set(multiValueArgs)

  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  if(NOT ARG_HTML)
    set(ARG_HTML NO)
  endif()
  if(NOT ARG_LATEX)
    set(ARG_LATEX NO)
  endif()
  if(NOT ARG_XML)
    set(ARG_XML YES)
  endif()
  if(NOT ARG_LANGUAGE)
    set(ARG_LANGUAGE "Russian")
  endif()
  if(NOT ARG_DOXYFILE_TEMPLATE OR NOT EXISTS "${ARG_DOXYFILE_TEMPLATE}")
    set(ARG_DOXYFILE_TEMPLATE "${CMAKE_CURRENT_FUNCTION_LIST_DIR}/Doxyfile.in")
  endif()
  if(NOT DOXYGEN_PROJECT_NAME)
    set(DOXYGEN_PROJECT_NAME ${PROJECT_NAME})
  endif()

  set(work_dir ${PROJECT_BINARY_DIR}/doc/doxygen)
  configure_file(${ARG_DOXYFILE_TEMPLATE} ${work_dir}/Doxyfile @ONLY)

  add_custom_target(${target}
    VERBATIM
    WORKING_DIRECTORY ${work_dir}
    COMMAND ${DOXYGEN_EXECUTABLE} ${work_dir}/Doxyfile
    COMMENT "Generating API documentation with Doxygen")
  if(ARG_HTML)
    install(DIRECTORY ${CMAKE_BINARY_DIR}/doc/doxygen/html/
            COMPONENT DOC OPTIONAL
            DESTINATION ${CMAKE_INSTALL_DATADIR}/doc/doxygen)
  endif()
  if(NOT TARGET myx-doc-doxygen)
    add_custom_target(myx-doc-doxygen)
  endif()
  add_dependencies(myx-doc-doxygen ${target})
endfunction()


include(${MYX_CMAKE_LIB_DIR}/generators/PrivateConfigHeader.cmake)
include(${MYX_CMAKE_LIB_DIR}/generators/GitInfoHeader.cmake)

function(myx_create_packages TARGET_NAME)
  if(NOT ${PROJECT_BINARY_DIR} STREQUAL ${CMAKE_BINARY_DIR})
    return()
  endif()

  set(options)
  set(oneValueArgs DEBIAN_PACKAGE_TYPE CPACK_DEBIAN_PACKAGE_SECTION CPACK_DEBIAN_PACKAGE_PRIORITY
                   CMAKE_INSTALL_DEFAULT_COMPONENT_NAME CPACK_PACKAGE_CONTACT CPACK_PACKAGING_INSTALL_PREFIX)
  set(multiValueArgs CPACK_SOURCE_GENERATOR CPACK_GENERATOR CPACK_SOURCE_IGNORE_FILES)
  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})

  # Общие настройки для пакета: организация, автор, версия
  if(NOT ${PROJECT_NAME}_CONTACT)
    if(${PROJECT_NAME}_AUTHOR_NAME AND ${PROJECT_NAME}_AUTHOR_EMAIL)
      set(CPACK_PACKAGE_CONTACT "${${PROJECT_NAME}_AUTHOR_NAME} <${${PROJECT_NAME}_AUTHOR_EMAIL}>")
    else()
      set(CPACK_PACKAGE_CONTACT "unknown maintainer <nonexistent@mailbox>")
      myx_message_warning("\${PROJECT_NAME}_CONTACT variable is required for packaging but unset")
    endif()
  else()
    set(CPACK_PACKAGE_CONTACT ${${PROJECT_NAME}_CONTACT})
  endif()

  if(NOT ${PROJECT_NAME}_VENDOR)
    if(${PROJECT_NAME}_AUTHOR_NAME)
      set(CPACK_PACKAGE_VENDOR "${${PROJECT_NAME}_AUTHOR_NAME}")
    else()
      set(CPACK_PACKAGE_VENDOR "unknown vendor")
      myx_message_warning("\${PROJECT_NAME}_VENDOR variable is required for packaging but unset")
    endif()
  else()
    set(CPACK_PACKAGE_VENDOR ${${PROJECT_NAME}_VENDOR})
  endif()

  string(TOLOWER ${PROJECT_NAME} PN)
  set(CPACK_PACKAGE_NAME ${PN})
  set(CPACK_PACKAGE_VERSION ${PROJECT_VERSION})

  if(NOT CPACK_PACKAGING_INSTALL_PREFIX)
    set(CPACK_PACKAGING_INSTALL_PREFIX "/usr")
  endif()

  # Параметры для архива исходных текстов
  if(NOT ARG_CPACK_SOURCE_GENERATOR)
    set(CPACK_SOURCE_GENERATOR "TXZ")
  else()
    set(ARG_CPACK_SOURCE_GENERATOR ${CPACK_SOURCE_GENERATOR})
  endif()
  set(CPACK_SOURCE_PACKAGE_FILE_NAME "${PN}-${CPACK_PACKAGE_VERSION}")

  # Типы генераторов для бинарных архивов
  if(NOT ARG_CPACK_GENERATOR)
    set(CPACK_GENERATOR "TXZ" "DEB")
  else()
    set(CPACK_GENERATOR ${ARG_CPACK_GENERATOR})
  endif()
  if(CMAKE_SYSTEM_NAME STREQUAL Windows)
    list(REMOVE_ITEM CPACK_GENERATOR "DEB")
  endif()

  # Параметры для архива собранного проекта
  set(CPACK_TARGET_ARCH ${CMAKE_SYSTEM_PROCESSOR})
  if(CMAKE_SYSTEM_PROCESSOR STREQUAL "x86_64")
    set(CPACK_TARGET_ARCH "amd64")
  endif()
  set(CPACK_PACKAGE_FILE_NAME "${PN}_${CPACK_PACKAGE_VERSION}")

  # Список масок для исключения из архива исходных текстов
  set(CPACK_SOURCE_IGNORE_FILES
      ${ARG_CPACK_SOURCE_IGNORE_FILES}
      ${CMAKE_BINARY_DIR}
      "^${CMAKE_SOURCE_DIR}/.?build.?/"
      "^${CMAKE_SOURCE_DIR}/.?output.?/"
      "^${CMAKE_SOURCE_DIR}/files/var"
      "^${CMAKE_SOURCE_DIR}/files/log"
      "CMakeLists.txt.user.*"
      ".*.autosave"
      ".*.status"
      "~$"
      "\\\\.swp$")

  option(MYX_COMPACT_SOURCE_PACKAGE "Make compact source package" ON)
  if(MYX_COMPACT_SOURCE_PACKAGE)
    # Список масок для исключения из архива исходных текстов для более компактного архива
    set(CPACK_SOURCE_IGNORE_FILES
        ${CPACK_SOURCE_IGNORE_FILES}
        "\\\\.git"
        "/\\\\.git/"
        "/\\\\.gitlab-ci/"
        "\\\\.clang-tidy$"
        "\\\\.cmake-format$"
        "\\\\.gitignore$"
        "\\\\.gitattributes$"
        "\\\\.gitmodules$"
        "\\\\.gitlab-ci.yml$")
  endif()

  if("TXZ" IN_LIST CPACK_GENERATOR)
    set(CPACK_ARCHIVE_COMPONENT_INSTALL OFF)
    set(CPACK_ARCHIVE_FILE_NAME "${PN}_${CPACK_TARGET_ARCH}_${CPACK_PACKAGE_VERSION}")
  endif()

  if("DEB" IN_LIST CPACK_GENERATOR)
    set(CPACK_DEB_MONOLITHIC_INSTALL OFF)
    set(CPACK_DEBIAN_PACKAGE_SHLIBDEPS ON)

    # По умолчанию пакет собирается для дистрибутива unstable
    if(NOT ARG_DEBIAN_PACKAGE_TYPE)
      set(DEBIAN_PACKAGE_TYPE "unstable")
    endif()

    if(NOT ARG_CPACK_DEBIAN_PACKAGE_SECTION)
      set(CPACK_DEBIAN_PACKAGE_SECTION "misc")
    endif()

    if(NOT ARG_CPACK_DEBIAN_PACKAGE_PRIORITY)
      set(CPACK_DEBIAN_PACKAGE_PRIORITY "optional")
    endif()

    # По умолчанию пакет для Debian делится на компоненты
    if(NOT ARG_CPACK_DEB_COMPONENT_INSTALL)
      set(CPACK_DEB_COMPONENT_INSTALL ON)
    endif()

    # Если имя компонента по умолчанию не определено, то устанавливается main
    if(NOT CMAKE_INSTALL_DEFAULT_COMPONENT_NAME)
      set(CMAKE_INSTALL_DEFAULT_COMPONENT_NAME MAIN)
    endif()

    # В списке компонентов обязательно должны быть main, lib, dev, static и doc
    #list(APPEND CPACK_COMPONENTS_ALL main lib dev static doc)
    # TODO
    #list(REMOVE_DUPLICATES CPACK_COMPONENTS_ALL)

    set(deb_arch_suffix "${CPACK_PACKAGE_VERSION}_${CPACK_TARGET_ARCH}.deb")
    # Правило формирования имени пакета и файла для компонента main
    set(CPACK_DEBIAN_MAIN_PACKAGE_NAME "${PN}")
    set(CPACK_DEBIAN_MAIN_FILE_NAME ${CPACK_DEBIAN_MAIN_PACKAGE_NAME}_${deb_arch_suffix})

    # Правило формирования имени пакета и файла для компонента lib
    set(CPACK_DEBIAN_LIB_PACKAGE_NAME "lib${PN}${PROJECT_VERSION_MAJOR}")
    set(CPACK_DEBIAN_LIB_FILE_NAME ${CPACK_DEBIAN_LIB_PACKAGE_NAME}_${deb_arch_suffix})

    # Правило формирования имени пакета и файла для компонента dev
    set(CPACK_DEBIAN_DEV_PACKAGE_NAME "lib${PN}-dev")
    set(CPACK_DEBIAN_DEV_FILE_NAME ${CPACK_DEBIAN_DEV_PACKAGE_NAME}_${deb_arch_suffix})

    # Правило формирования имени пакета и файла для компонента static
    set(CPACK_DEBIAN_STATIC_PACKAGE_NAME "lib${PN}${PROJECT_VERSION_MAJOR}-static-dev")
    set(CPACK_DEBIAN_STATIC_FILE_NAME ${CPACK_DEBIAN_STATIC_PACKAGE_NAME}_${deb_arch_suffix})
    set(CPACK_DEBIAN_STATIC_PACKAGE_DEPENDS "lib${PN}-dev")

    # Правило формирования имени пакета и файла для компонента doc
    set(CPACK_DEBIAN_DOC_PACKAGE_NAME "${PN}-doc")
    set(CPACK_DEBIAN_DOC_FILE_NAME ${PN}-doc_${CPACK_PACKAGE_VERSION}_all.deb)

    foreach(iter ${CPACK_COMPONENTS_ALL})
      string(TOLOWER ${iter} _cl)
      string(TOUPPER ${iter} _cu)
      # Правила формирования имени пакета и файла для остальных компонентов
      if((NOT ${_cu} STREQUAL MAIN) AND
         (NOT ${_cu} STREQUAL LIB) AND
         (NOT ${_cu} STREQUAL DEV) AND
         (NOT ${_cu} STREQUAL STATIC) AND
         (NOT ${_cu} STREQUAL DOC))
        set(CPACK_DEBIAN_${_cu}_PACKAGE_NAME "${PN}-${_cl}")
        set(CPACK_DEBIAN_${_cu}_FILE_NAME "${CPACK_DEBIAN_${_cu}_PACKAGE_NAME}_${deb_arch_suffix}")
      endif()

      # Если в каталоге ${CMAKE_SOURCE_DIR}/cmake/deb/${_cl} находятся сценарии сопровождающего
      # postinst, preinst, postrm и prerm, то они будут добавлены к пакету.
      if(EXISTS "${CMAKE_SOURCE_DIR}/cmake/deb/${_cl}/preinst")
        list(APPEND CPACK_DEBIAN_${_cu}_PACKAGE_CONTROL_EXTRA "${CMAKE_SOURCE_DIR}/cmake/deb/${_cl}/preinst")
      endif()
      if(EXISTS "${CMAKE_SOURCE_DIR}/cmake/deb/${_cl}/postinst")
        list(APPEND CPACK_DEBIAN_${_cu}_PACKAGE_CONTROL_EXTRA "${CMAKE_SOURCE_DIR}/cmake/deb/${_cl}/postinst")
      endif()
      if(EXISTS "${CMAKE_SOURCE_DIR}/cmake/deb/${_cl}/prerm")
        list(APPEND CPACK_DEBIAN_${_cu}_PACKAGE_CONTROL_EXTRA "${CMAKE_SOURCE_DIR}/cmake/deb/${_cl}/prerm")
      endif()
      if(EXISTS "${CMAKE_SOURCE_DIR}/cmake/deb/${_cl}/postrm")
        list(APPEND CPACK_DEBIAN_${_cu}_PACKAGE_CONTROL_EXTRA "${CMAKE_SOURCE_DIR}/cmake/deb/${_cl}/postrm")
      endif()
    endforeach()

    if(UNIX AND NOT TARGET deb)
      add_custom_target(deb WORKING_DIRECTORY ${CMAKE_BINARY_DIR} COMMAND cpack -G DEB)
      add_dependencies(deb ${TARGET_NAME})
    endif()
  endif()

  # Подключение модуля, выполняющего сборку архивов и пакетов
  include(CPack)
endfunction(myx_create_packages TARGET_NAME)

#[=======================================================================[.rst:
Цель для удаления файлов, установленных выполнением цели `install`.
Если при установке использовалась переменная `DESTDIR`, то при удалении
файлов нужно указать то же значение.
#]=======================================================================]

include_guard(GLOBAL)

if((NOT TARGET uninstall) AND (NOT DEFINED ENV{SAPR_PREFIX}))
  configure_file(${CMAKE_CURRENT_LIST_DIR}/uninstall.cmake.in
                 ${CMAKE_BINARY_DIR}/cmake_uninstall.cmake IMMEDIATE @ONLY)
  add_custom_target(uninstall COMMAND ${CMAKE_COMMAND} -P ${CMAKE_BINARY_DIR}/cmake_uninstall.cmake)
endif()


unset(MYX_CMAKE_SOURCE_DIR)
unset(MYX_CMAKE_BACKPORTS_DIR)
unset(MYX_CMAKE_LIB_DIR)

cmake_policy(POP)
